#!/usr/bin/env python

import sys
import os
import shutil
import socket
import string
import random
import json
import collections

from nacoma.hooks import Change

bps_path = '/opt/synergy/etc/bps/'
if len(os.listdir(bps_path)) == 0:
    sys.exit(1)  # exit if there no bps

reportable_types = ['host', 'service', 'hostgroup', 'servicegroup']
synergy_socket_path = '/opt/monitor/var/rw/synergy'

d = ''

def create_work_dir():
    global d
    if os.path.exists(d): return

    while True:
        d = '/opt/synergy/var/queue/%s' % (''.join(random.choice(string.ascii_uppercase + string.digits) for x in range(6)))
        if not os.path.exists(d): break

    old_umask = os.umask(0)
    shutil.copytree(bps_path, d) # copy bps for processing
    os.chmod(d, 0o775)
    for file in os.listdir(d):
        f = os.path.join(d, file)
        os.chmod(f, 0o664)

    os.umask(old_umask)


def rule_str(name, obj, type, rep):
    return 'name = \"%s\",\\n%s\\t%sintent = nagios.%s(\"%s\"' % (name, rep, rep, type,  obj)


def replace_host(old, new):
    os.system("sed -i ':a;N;$!ba;s|%s|%s|g;s|\"%s\&|\"%s\&|g;s| on %s\"| on %s\"|g;s|host_services(\"%s\"|host_services(\"%s\"|g;s|host = \"%s\"|host = \"%s\"|g' %s/*.lua" % (rule_str(old, old, 'host', '*'), rule_str(new, new, 'host', ""), old, new, old, new, old, new, old, new, d))


def delete_materialized_host(host_name):
    search_str = ",\\s*\\t*\\n*materialized\\n*\\s*\\t*=\\n*\\s*\\t*{\\n*\\s*\\t*host\\n*\\s*\\t*=\\n*\\s*\\t*\"%s\"\\n*\\s*\\t*}" % (host_name)
    os.system("sed -i ':a;N;$!ba;s!%s!!g' %s/*.lua" % (search_str, d))

def query_synergy_socket(query):
    sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
    sock.connect(synergy_socket_path)
    sock.send(query)
    response = ''
    bufsize = 2048
    while True:
        read = sock.recv(bufsize)
        if read == '':
            break
        response += read
    sock.close()
    return response

def get_materialized_services():
    response = query_synergy_socket("full\n")
    try:
        json_response = json.loads(response)
    except ValueError:
        return {}
    services = collections.defaultdict(list)
    for i in json_response['items']:
        if 'materialized' in json_response['items'][i]:
            services[json_response['items'][i]['materialized']['host']].append(json_response['items'][i]['name'])
    return services

def service_obj(name):
    return (name.split(';')[0]+'\&'+name.split(';')[1])


def service_name(name):
    return name.split(';')[1]+' on '+name.split(';')[0]


def replace_service(old, new):
    os.system("sed -i ':a;N;$!ba;s!%s!%s!g' %s/*.lua" % (rule_str(service_name(old), service_obj(old), "service", "*"), rule_str(service_name(new), service_obj(new), "service", ""), d))


def replace_hg(old, new):
    os.system("sed -i ':a;N;$!ba;s!%s!%s!g' %s/*.lua" % (rule_str(old, old, "host_group", "*"), rule_str(new, new, "host_group", ""), d))


def replace_sg(old, new):
    os.system("sed -i ':a;N;$!ba;s!%s!%s!g' %s/*.lua" % (rule_str(old, old, "service_group", "*"), rule_str(new, new, "service_group", ""), d))

def trigger_synergy_to_load_spool_files():
    response = query_synergy_socket("forceloop\n")
    # Do something with response if you want to debug, such as:
    #with open('/tmp/kungenochsilvia', 'w') as f:
        #f.write(response)

def main():
    materialized_services = get_materialized_services()
    printed = False
    for line in sys.stdin:
        change = Change(line)
        if change.type not in reportable_types:
            continue
        if change.is_renamed():
            if change.type == 'host':
                create_work_dir()
                replace_host(change.oldname, change.newname)
            if change.type == 'service':
                create_work_dir()
                replace_service(change.oldname, change.newname)
                if not printed:
                    print('<span style="font-weight: bold;">')
                    print('If the renamed service represents a <a href="/monitor/index.php/synergy/view" target="_parent">Business Service</a>, this needs to be manually updated.<br />')
                    print('<br /></span>')
                    printed = True
            if change.type == 'hostgroup':
                create_work_dir()
                replace_hg(change.oldname, change.newname)
            if change.type == 'servicegroup':
                create_work_dir()
                replace_sg(change.oldname, change.newname)
        if change.is_deleted():
            if change.type == 'host':
                create_work_dir()
                delete_materialized_host(change.oldname)
            if change.type == 'service' and len(materialized_services) > 0:
                h, s = change.oldname.split(";")
                if s in materialized_services.get(h, []):
                    print('<span style="font-weight: bold;">')
                    print('The deleted service <em>%s</em> represented a <a href="/monitor/index.php/synergy/view" target="_parent">Business Service</a>, which needs to be manually updated.<br />' % (change.oldname))
                    print('<br /></span>')
    if not os.path.exists(d):
        sys.exit()

    shutil.move(d, "/opt/synergy/var/spool/")
    trigger_synergy_to_load_spool_files()

if __name__ == '__main__':
    main()
