<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Synergy controller
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Synergy_Controller extends Ninja_Controller
{
	const MODE_VIEW = 1;
	const MODE_EDIT = 2;

	protected function _initHead()
	{
		$this->template->content->authorized_for_edit = nacoma::allowed();
		// This might be used later. Class nacoma is located in nacoma helper in ninja.
		// $this->template->content->authorized_for_export = nacoma::exportAllowed();
		$this->template->js_header = $this->add_view('js_header');
		try {
			$this->template->js_strings = "var synergy_data = " . json_encode(Synergy_Model::getStates()) . ";\n var actions = " . Synergy_Model::getActionsJson() . ";\n";
			$this->template->js_strings .= "var nagios = ".Synergy_Model::getNagiosObjectsJson().";\n";
		} catch(SynergySocketException $e) {
			$this->template->content = $this->add_view('synergy/error');
			$this->template->content->title = _("There was an error");
			$this->template->content->error = $e->getMessage();
			return;
		}
		$this->template->css[] = 'modules/synergy/views/synergy/css/synergy.css';
		$synergy_js_path = 'modules/synergy/js/';
	}

	public function view()
	{
		$this->_verify_access('monitor.bsm.tree:read.gui');

		$this->template->content = $this->add_view('synergy/index');
		$this->template->content->mode = self::MODE_VIEW;
		$this->_initHead();
		$this->template->title = _('Monitoring » Business Services » View');
		$this->template->disable_refresh = true;
	}

	public function edit()
	{
		$this->_verify_access('monitor.bsm.tree:create.gui');
		$this->_verify_access('monitor.bsm.tree:update.gui');
		$this->_verify_access('monitor.bsm.tree:delete.gui');

		$this->template->content = $this->add_view('synergy/index');
		$this->template->content->mode = self::MODE_EDIT;
		$this->_initHead();
		$this->template->title = _('Monitoring » Business Services » Edit');
		$this->template->disable_refresh = true;
	}

	/**
	 * Fetches states for all trees
	 */
	public function update()
	{
		$this->_verify_access('monitor.bsm.tree:update.gui');

		$result = array(
			"status" => 200,
			"data" => array()
		);
		$trees = Synergy_Model::getStates();
		$result["data"]["tree"] = $trees;
		return json::ok($result);
	}

	/**
	 * Checks whether rules exist
	 * @return string
	 */
	public function rule_check()
	{
		$this->_verify_access('monitor.bsm.tree:read.gui');

		$this->auto_render = false;
		$result = array("status" => 200, "data" => false);
		try {
			if (!isset($_GET["rule"])) {
				throw new Exception("You have to pass a rule argument");
			}
			$rule = (string) $_GET["rule"];
			if (strlen($rule) === 0) {
				throw new Exception("You have to pass a non-empty rule argument");
			}
			$result["data"] = Synergy_Model::ruleExists($rule);

		} catch(Exception $e) {
			$result["data"] = $e->getMessage();
			$result["status"] = 503;
		}
		return json::ok($result);
	}
	/**
	 *	Checks whether a service exists or not
	 *	@return string
	 */
	public function check()
	{
		$this->_verify_access('monitor.bsm.tree:read.gui');

		$this->auto_render = false;
		$result = array("status" => 200, "data" => false);
		try {
			if (!isset($_GET["service"])) {
				throw new Exception("You have to pass a service argument");
			}
			if (strpos($_GET["service"], '&') === false) {
				throw new Exception("Illegal service argument");
			}
			$result['data'] = false !== ServicePool_Model::fetch_by_key(str_replace('&', ';', $_GET['service']));
		} catch(Exception $e) {
			$result["data"] = $e->getMessage();
			$result["status"] = 503;
		}
		return json::ok($result);
	}

	/**
	 * A user pressed "save" in the GUI.
	 */
	public function sync()
	{
		$this->_verify_access('monitor.bsm.tree:update.gui');

		if (nacoma::allowed() === false) {
			header("HTTP/1.0 401 Unauthorized");
			exit;
		}
		$this->auto_render = false;
		try {
			$nacomaapi = new NacomaApi_Model();

			$uuid = uniqid();
			if (isset($_POST["update_list"])) {
				$update_list = json_decode($_POST["update_list"], true);
				if($update_list === null) {
					throw new Exception("'update_list' argument does not contain valid JSON");
				}
				foreach ($update_list as $rule_name => $rule_json) {
					// throws exception on failure
					Synergy_Model::saveRule($rule_json, $rule_name, $uuid);
				}
			}

			if (isset($_POST["del_list"])) {
				// throws exception on failure
				Synergy_Model::deleteRulesList($_POST["del_list"], $uuid);
			}
			// throws exception on failure
			Synergy_Model::commitActions($uuid);

			if (isset($_POST["add"]) or isset($_POST["del"]) or isset($_POST["rename"])) {
				if (isset($_POST["add"])) {
					$add = json_decode($_POST["add"], true);
					// throws exception on failure
					$nacomaapi->create_services($add);
				}

				if (isset($_POST["del"])) {
					$del = json_decode($_POST["del"], true);
					// throws exception on failure
					$nacomaapi->delete_services($del);
				}

				if (isset($_POST["rename"])) {
					$ren = json_decode($_POST["rename"], true);
					// throws exception on failure
					$nacomaapi->rename_services($ren);
				}
				// throws exception on failure
				$nacomaapi->saveconfig();
			}
		} catch (Exception $e) {
			// "ok", because synergy uses the result key to determine whether
			// the request worked or not, while the json helper assumes HTTP is
			// good enough.
			return json::ok(array("result" => false, "msg" => $e->getMessage()));
		}

		return json::ok(array("result" => true, "timestamp" => time()));
	}
}
