<?php defined('SYSPATH') OR die('No direct access allowed.');

class Synergy_report_Controller extends Authenticated_Controller
{

	private static function log($level, $msg) {
		require_once('op5/log.php');
		$log = Op5Log::instance('synergy');
		$log->log($level, $msg);
	}

	/**
	 * Insert synergy report data to database
	 *
	 * Listens for JSON streamed on STDIN, something like:
	 * @code
	 * {
	 * 	'meta': {
	 * 		'timestamp': x,
	 * 		'host': y,
	 * 		'name': z,
	 * 		'state': 0
	 * 	},
	 * 	'data':  {
	 * 		'items': {
	 * 		... bunch of synergy data here ...
	 * 		}
	 * 	}
	 * }
	 * @endcode
	 */
	public function insert_report_data() {
		if(PHP_SAPI !== "cli") {
			$this->log("error", "Not CLI");
			echo "You can only request ".__METHOD__." from cli";
			exit(2);
		}
		$this->auto_render = false;
		$json = "";
		$in = fopen('php://stdin', 'r');
		while(!feof($in)) {
			$json = $json . fgets($in, 102400);
		}
		$tree = json_decode($json);
		if($tree === null) {
			$json_errno = json_last_error();
			$this->log("error", "Got invalid report data from synergy (json decode errno: '$json_errno'): ".var_export($json, true));
			return;
		}
		if(!isset($tree->meta)) {
			$this->log("error", "Missing metadata in report data from synergy: ".var_export($json, true));
			return;
		}
		$timestamp = $tree->meta->timestamp;
		$host = $tree->meta->host;
		$service = $tree->meta->name; // notice that the service name is always 1:1 to the BSM object's name
		$state = $tree->meta->state;
		unset($tree->meta);
		$db = Database::instance();
		if (!$tree->data) {
			// Recieved an OK state represented by empty string. Setting database entry to NULL
			$json = "NULL";
		} else {
			$json = json_encode($tree->data);
			$json = $db->escape($json);
		}
		$this->log('debug', 'Storing Synergy Report data for rule: ' . $service);
		$sql = "INSERT INTO report_data_synergy(timestamp, event_type, host_name, service_description, state, tree)" .
		" VALUES(".$db->escape($timestamp).", 701, ".$db->escape($host).", ".$db->escape($service).", ".$db->escape($state).", " . $json . ")";
		$db->query($sql);
	}
}
